using System.Collections.Generic;
using System.ComponentModel;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Grid.Tutorial.BusinessData.CAC40
{
    public enum FeederState
    {
        Ok,
        PartialOk,
        Nok,
    }

    public class Feeder : INotifyPropertyChanged
    {
        #region Constants

        public const int MaxHistoryLength = 40;

        public const string FieldAsk = "Ask";
        public const string FieldAskQty = "AskQty";
        public const string FieldBid = "Bid";
        public const string FieldBidQty = "BidQty";
        public const string FieldMid = "Mid";
        public const string FieldClose = "Close";
        public const string FieldHi = "Hi";
        public const string FieldHistory = "History";
        public const string FieldInstrument = "InstrumentName";
        public const string FieldLast = "Last";
        public const string FieldLastQty = "LastQty";
        public const string FieldLow = "Low";
        public const string FieldMarket = "MarketName";
        public const string FieldOpen = "Open";
        public const string FieldVolume = "Volume";
        public const string FieldTurnover = "Turnover";
        public const string FieldChange = "Change";
        public const string FieldState = "State";

        #endregion

        #region Fields

        private readonly List<double> _history = new List<double>();
        private readonly Instrument _instrument;
        private readonly Market _market;
        private double _ask;
        private long _askQty;
        private double _bid;
        private long _bidQty;
        private double _close;
        private double _hi;
        private double _last;
        private long _lastQty;
        private double _low;
        private double _open;
        private double _prevAsk;
        private double _prevBid;
        private double _prevLast;
        private FeederState _state = FeederState.Ok;
        private double _volume;

        #endregion

        public Feeder(Instrument instrument, Market market, double last, long lastQty, double volume, double close, double open, double hi, double low)
        {
            _instrument = instrument;
            _market = market;
            _last = last;
            _volume = volume;
            _lastQty = lastQty;
            _close = close;
            _open = open;
            _hi = hi;
            _low = low;
        }

        [Field(FieldState)]
        public FeederState State
        {
            get { return _state; }
            set
            {
                if (_state != value)
                {
                    _state = value;
                    Notify(FieldState);
                }
            }
        }

        [CompositeField]
        public Market Market
        {
            get { return _market; }
        }

        [CompositeField]
        public Instrument Instrument
        {
            get { return _instrument; }
        }


        [Field(FieldMid)]
        public double Mid
        {
            get { return (_bid + _ask)/2; }
        }

        [Field(FieldBid)]
        [DoubleFormat(3)]
        public double Bid
        {
            get { return _bid; }
            set
            {
                _prevBid = _bid;
                if (_bid != value)
                {
                    _bid = value;
                    Notify(FieldBid);
                    Notify(FieldMid);
                }
            }
        }

        [Field(FieldChange)]
        [Format("0.##", "", " %")]
        public double Change
        {
            get { return _open > 0 ? 100*((_last - _open)/_open) : 0; }
        }

        [Field(FieldAsk)]
        [DoubleFormat(3)]
        public double Ask
        {
            get { return _ask; }
            set
            {
                _prevAsk = _ask;
                if (_ask != value)
                {
                    _ask = value;
                    Notify(FieldAsk);
                    Notify(FieldMid);
                }
            }
        }

        [Field(FieldBidQty)]
        public long BidQty
        {
            get { return _bidQty; }
            set
            {
                if (_bidQty != value)
                {
                    _bidQty = value;
                    Notify(FieldBidQty);
                }
            }
        }

        [Field(FieldAskQty)]
        public long AskQty
        {
            get { return _askQty; }
            set
            {
                if (_askQty != value)
                {
                    _askQty = value;
                    Notify(FieldAskQty);
                }
            }
        }

        [Field(FieldLast)]
        [DoubleFormat(3)]
        public double Last
        {
            get { return _last; }
            set
            {
                _prevLast = _last;

                _history.Insert(0, value);
                if (_history.Count > MaxHistoryLength)
                {
                    _history.RemoveAt(MaxHistoryLength);
                }
                Notify(FieldHistory);

                if (_last != value)
                {
                    _last = value;
                    Notify(FieldLast);
                    Notify(FieldChange);
                }

                if (_last > _hi)
                {
                    _hi = _last;
                    Notify(FieldHi);
                }
                if (_last < _low)
                {
                    _low = _last;
                    Notify(FieldLow);
                }
            }
        }

        [Field(FieldLastQty)]
        public long LastQty
        {
            get { return _lastQty; }
            set
            {
                Volume += value;
                if (_lastQty != value)
                {
                    _lastQty = value;
                    Notify(FieldLastQty);
                }
            }
        }

        [Field(FieldClose)]
        public double Close
        {
            get { return _close; }
            set { _close = value; }
        }

        [Field(FieldOpen)]
        public double Open
        {
            get { return _open; }
            set { _open = value; }
        }

        [Field(FieldHi)]
        [DoubleFormat(3)]
        public double Hi
        {
            get { return _hi; }
            set { _hi = value; }
        }

        [Field(FieldLow)]
        [DoubleFormat(3)]
        public double Low
        {
            get { return _low; }
            set { _low = value; }
        }

        [Field(FieldVolume)]
        [DoubleFormat(ShortForm = true, Precision = 2)]
        public double Volume
        {
            get { return _volume; }
            set
            {
                if (_volume != value)
                {
                    _volume = value;
                    Notify(FieldVolume);
                }
            }
        }

        [Field(FieldHistory)]
        public IList<double> History
        {
            get { return _history; }
        }

        public double PrevBid
        {
            get { return _prevBid; }
        }

        public double PrevAsk
        {
            get { return _prevAsk; }
        }

        public double PrevLast
        {
            get { return _prevLast; }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        private void Notify(string fieldId)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(fieldId));
            }
        }
    }
}